#!/usr/local/bin/perl -w

%data = ();
%uids = ("batt", 0,
	 "mainin", 0,
	 "corein", 0,
	 "core", 0,
	 "digital", 0,
	 "dram", 0,
	 "analog", 0);

sub usage {
    warn(
	 "Usage: calceff.pl -dontround   [<filename>...]+\n");
    exit 1;
}

sub arguments {
    $dontround = 1;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-dontround", 0, length $_)) {
		$dontround = 1;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}

sub pline {
    local($itsy) = shift;
    local($tmp) = shift;
    local($Pavg) = shift;
    local($Rerr) = shift;
    printf "itsy \#$itsy, %28s ", $tmp;
    printf 
	"Pavg %.${p}f mW, RErr %.${p}f%%\n", $Pavg, $Rerr;
}

sub pline_diff {
    local($itsy) = shift;
    local($tmp) = shift;
    local($Pavg) = shift;
    local($Percent) = shift;
    local($Foo) = shift;
    printf "itsy \#$itsy, %40s ", $tmp;
    printf 
	"Pavg %.${p}f mW, Percent %.${p}f%%    %s\n", $Pavg, $Percent, $Foo;
}

sub calcrerr {
    local($tmp) = shift;
    local($Pavg1) = shift;
    local($Rerr1) = shift;
    local($Pavg2) = shift;
    local($Rerr2) = shift;
    local($PavgNew) = shift;
    local($RerrNew);
    if ($Pavg1+$Pavg2 != 0) {
	$RerrNew = ($Rerr1*abs($Pavg1)+$Rerr2*abs($Pavg2))/abs($PavgNew);
    }
    else {
	$RerrNew = 1000000000;
	printf STDERR "Warning: $tmp Rerr faked due to zero power\n";
    }
    $RerrNew;	
}

sub processit { # IN_FILEHANDLE 
    local($IN1) = shift;

    $p = 6;
    $p = 3 if (!$dontround);

    $currentcommand = "";
    foreach $uid (%uids) {
	$data{$uid} = "";
    }
    while(<$IN1>) {
	next if (/mismatch/);
	next if (/0 voltage measured/);
	next if (/dict/);
	($itsy, $command, $condition, $uid, $Pavg, $Rerr) = 
	    $_ =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+/;
#	print "$itsy, $command, $condition, $uid, $Pavg, $Rerr\n";
	next if ($uid eq "batt");

	$currentcommand = $command if (! $currentcommand);
	if ($command ne $currentcommand) {
	    printf
		"%50s  %7s   %7s   Eff: %5s%%\n", 
		sprintf("%s: %s-%s", $infile1, $currentcommand, $condition),
		sprintf("%4.2f", $data{"mainin"}+$data{"corein"}),
		sprintf("%4.2f", $data{"analog"}+$data{"dram"}+$data{"digital"}+$data{"core"}),
		sprintf("%3.1f", 100*(($data{"analog"}+$data{"dram"}+$data{"digital"}+$data{"core"})/($data{"mainin"}+$data{"corein"})));
	    foreach $uid (%uids) {
		$data{$uid} = "";
	    }
	    $currentcommand = $command;
	}
	$data{$uid} = $Pavg;
    }
    printf
	"%50s  %7s   %7s   Eff: %5s%%\n", 
	sprintf("%s: %s-%s", $infile1, $currentcommand, $condition),
	sprintf("%4.2f", $data{"mainin"}+$data{"corein"}),
	sprintf("%4.2f", $data{"analog"}+$data{"dram"}+$data{"digital"}+$data{"core"}),
	sprintf("%3.1f", 100*(($data{"analog"}+$data{"dram"}+$data{"digital"}+$data{"core"})/($data{"mainin"}+$data{"corein"})));
}

&arguments();

FILE1:

while ($#ARGV >= 0) {
    $infile1 = shift @ARGV;
    open(FILE1, $infile1) || (die "Can't open $infile1: $!\n");
    
    processit(FILE1);
    close(FILE1);
}

