#!/usr/local/bin/perl -w

sub usage {
    warn(
	 "Usage: combine.pl -dontround [<filename>...]+\n");
    exit 1;
}

sub arguments {
    $dontround = 0;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-dontround", 0, length $_)) {
		$dontround = 1;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}


%results = ();

sub processit { # IN_FILEHANDLE 
    local($IN) = $_[0];

    while(<$IN>) {
	next if (/mismatch/);
	next if (/0 voltage measured/);
	($itsy, $command, $condition, $uid, $Pavg, $Rerr, $Plowbnd, $Pupbnd, $Tres) = 
	    $_ =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+low:(\S+)\s+up:(\S+)\s+Tres:(\S+)\s+/;
#	print "$itsy, $command, $condition, $uid, $Pavg, $Rerr, $Plowbnd, $Pupbnd, $Tres\n";

	$uids{$uid} = 1;
	$prefix = $command."-".$uid;
	$tmp = sprintf("$condition#%10.10f#%10.10f=", $Pavg, $Rerr);
	$results{$prefix} .= $tmp;
#	printf STDERR "set %s\n", $prefix;
    }
}

&arguments();

FILE:

if ($#ARGV < 0) {
    &usage();
}

#@fullcomplist = ("idle", "wav", "tts", "mpeg");
@complist = ("idle", "wav", "tts");
@speeds = ( 59, 74, 88, 103, 118, 133, 147, 162, 177, 192, 206);
@marks = ( "circle", "box", "diamond", "triangle", "x", "cross", "ellipse");
@colors = ("1 0 0", "0 1 0", "0 0 1", ".5 .5 0", ".4 0 .7", ".7 .2 .1", ".7 0 .5");

sub printgraphheader {
    local($volt) = $_[0];
    local($exp) = $_[1];
    local($speed);
    printf "newgraph\n";
    printf "xaxis no_auto_hash_marks\n";
    foreach $speed (@speeds) {
	next if (($exp eq "tts") && ($speed eq "59"));
	last if (($speed > 88) && ($volt eq "low"));
	printf "hash_at %d hash_label at %d : %d\n", $speed, $speed, $speed;
    }
}

while ($#ARGV >= 0) {
    $infile = shift @ARGV;
    open(FILE, $infile) || (die "Can't open $infile: $!\n");
    
    processit(FILE);
    close(FILE);

    # first prepare idle/wav/tts 59-206 high
    
    foreach $uid (keys %uids) {
	foreach $volt ("high", "low") {

	    $markindex = 0;

	    $filename = $uid."-".$volt.".jgr";
	    open(G, "> $filename") || (die "Can't open $filename: $!\n");
	    select(G);

	    printgraphheader($volt, "combo");

	    foreach $exp (@complist) {

		$first = 1;
		foreach $speed (@speeds) {
		    $tmp = $results{$exp.$speed.$volt."-".$uid};
		    if ($tmp) {
			if ($first) {
			    printf "newcurve marktype %s fill 1 color %s label : %s%s \npts\n", 
			    $marks[$markindex], $colors[$markindex], $exp, $volt;
			    $first = 0;
			    $markindex++;
			}
			@junk = split("\#", $tmp);
			printf "%d %f\n", $speed, $junk[1];
		    }
		}
	    }
	    close(G);
	}
    }

    # then by experiment (all uids)
    
    foreach $exp (@complist) {
	foreach $volt ("high", "low") {

	    $markindex = 0;

	    $filename = $exp."-".$volt.".jgr";
	    open(G, "> $filename") || (die "Can't open $filename: $!\n");
	    select(G);

	    printgraphheader($volt, $exp);

	    foreach $uid (keys %uids) {

		$first = 1;
		foreach $speed (@speeds) {
		    $tmp = $results{$exp.$speed.$volt."-".$uid};
		    if ($tmp) {
			if ($first) {
			    printf "newcurve marktype %s fill 1 color %s label : %s-%s%s \npts\n", 
			    $marks[$markindex], $colors[$markindex], $uid, $exp, $volt;
			    $first = 0;
			    $markindex++;
			}
			@junk = split("\#", $tmp);
			printf "%d %f\n", $speed, $junk[1];
		    }
		}
	    }
	    close(G);
	}
    }
}
