#!/usr/local/bin/perl -w

$echar_sleep = .020;  
$echar_notsleep = .080;
$nbint = 150;

#               name          Tres
%program_info = (
"sleep",           "sleep#"       . "3600",
"deepsleep",       "deepsleep#"   . "3600",
"sleeplcd",        "sleeplcd#"    . "3600",
"sleepdc",         "sleepdc#"     . "3600",
"mpeg206high",     "mpeg206high#" .  " 90",
"dectalk74low",    "tts74low#"    .  "195",
"dectalk74high",   "tts74high#"   .  "195",
"dectalk206high",  "tts206high#"  .  "195",
"wav59high",       "wav59high#"   .  "252",
"wav206high",      "wav206high#"  .  "252",
"idle59low",       "idle59low#"   .  "600",
"idle59high",      "idle59high#"  .  "600",
"idle133high",     "idle133high#" .  "600",
"idle206high",     "idle206high#" .  "600",
"natspeak206high", "dict206high#" .  "102",
);

@speeds = ( 59, 74, 88, 103, 118, 132, 147, 162, 177, 192, 206, 
	   221, 236, 265, 295, 310, 324, 354, 383, );

sub usage {
    warn("Usage: gpgin [-nbint <number>] [-forcemaxt <seconds>] <filename>+\n");
    exit 1;
}

sub arguments {
    $forcemaxt = "";
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-nbint", 0, length $_)) {
		$nbint = shift @ARGV;
	    } elsif ($_ eq substr("-forcemaxt", 0, length $_)) {
		$forcemaxt = shift @ARGV;
		$forcemaxt = "-forcemaxt " . $forcemaxt . " ";
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}

sub processit { # IN_FILEHANDLE 
    local($IN) = $_[0];
    local($pathname) = $_[1];

    $battery = 0;
    open(BFILE, "$pathname/STATS") || (die "Can't open $pathname/STATS: $!\n");
    while (<BFILE>) {
	if (/^Itsy \#\d+, battery\s+#(\d+)\D/) {
	    $battery = $1;
	    last;
	}
    }
    close(BFILE);

    while(<$IN>) {
	if (/^running command .\/doit  /) {
	    if (/^running command .\/doit\s+(\w+)\s+-voltage\s+(\d+)\s+-speed\s+(\d+) (.+)$/) {
		$prefix = $1;
		$command_index = sprintf("%s%dv%s", $prefix, $speeds[$3], $2);
		$rest = $4;
	    }
	    elsif (/^running command .\/doit  (\w+)\s+-speed\s+(\d+) -(\w+) (.+)$/) {
		$prefix = $1;
		$command_index = sprintf("%s%d%s", $prefix, $2, $3);
		$rest = $4;
	    }
	    elsif (/^running command .\/doit  (\w*sleep\w*)\s+-speed\s+(\d+) (.+)$/) {
		$prefix = $1;
		$command_index = sprintf("%s%d%s", $prefix, $2, "");
		$rest = $3;
	    }
	    elsif (/^running command .\/doit  (\w+)\s+-aspeed\s+(\d+) (.+)$/) {
		# faking low
		die "this script only works if aspeed was <= 88!\n" if ($speeds[$2] > 88);
		$prefix = $1;
		$command_index = sprintf("%s%d%s", $prefix, $speeds[$2], "low");
		$rest = $3;
	    }
            else {
		die "No match found!\n";
	    }
	    if ($prefix =~ /sleep/) {
		$command_index = $prefix;
	    }
	    if ($rest =~ /-noprint/) {
		$printstr = "-noprint";
	    }
	    $tmp = $program_info{$command_index};
	    if ($tmp) {
		@tmp = split(/\#/, $program_info{$command_index});
		$command = $tmp[0];
		$tres = $tmp[1];
	    }
	    else {
		$tres = -1;
		if ($command_index =~ /^dectalk(.+)$/) {
		    $command = "tts" . $1;
		}
		elsif ($command_index =~ /^natspeak(.+)$/) {
		    $command = "dict" . $1;
		} 
		else {
		    $command = $command_index;
		}
	    }
	    $nchars = 0;
	    @outfiles = ();
	}
	elsif (/^\w+\s+(\w+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+P.T (\d+):  \+\s*$/) {
	    # eventually will be absolute time last character was seen
	    $nchars++;
	}
	elsif ($command && (/labinstr.*\/($command\.[a-z]+\.\d+\.out) /)) {
	    push(@outfiles, $1);
	}
	elsif (/^Experiment seemed to take (\d+) seconds/) {
	    foreach $outfile (@outfiles) {

		($expname, $host, $junk2, $junk3) = split('\.', $outfile);
		$junk2 = $junk3;  # get rid of stupid warning

		if ($outfile =~ /sleep/) {
		    $echar = $echar_sleep;
		}
		else {
		    $echar = $echar_notsleep;
		}

		if (! $results{$expname.$host}) {
		    $results{$expname.$host} = 
			sprintf ("/udir/kerr/measurements/power/full/bin/pwrgraph " .
				 "-noround -tres %d -nchars %d -echar %f -nbint %d -host %s %s %s",
				 $tres, $nchars-1, $echar, $nbint, $host, $printstr, $forcemaxt);
		}
		$results{$expname.$host} .= sprintf("%s/%s ", $pathname, $outfile);
	    }


	}
    }

}

$printstr = "";

FILE:
if ($#ARGV < 0) {
    &usage();
}

arguments();

while ($#ARGV >= 0) {
    $filename = shift @ARGV;
#    open(FILE, $filename) || (die "Can't open $filename: $!\n");
    open(FILE, $filename) || last;
    
    # discard path
    @data = split('/', $filename);
    pop(@data);

    processit(FILE, join('/', @data));
    
    close(FILE);
}

sub bymine {  # $a, $b
    local($prefix_a, $num_a, $suffix_a, $host_a) = ("", "", "", "");
    local($prefix_b, $num_b, $suffix_b, $host_b) = ("", "", "", "");
    if ($a =~ /^(\D+)(\d+)(\D+)(teeny|bitsy|weeny)$/) {
	($prefix_a, $num_a, $suffix_a, $host_a) = ($1, $2, $3, $4);
    }
    elsif ($a =~ /^(\D+)(teeny|bitsy|weeny)$/) {
	($prefix_a, $host_a) = ($1, $2);
    }
    elsif ($a =~ /^(\D+)(\d+)v(\d+)(teeny|bitsy|weeny)$/) {
	($prefix_a, $num_a, $suffix_a, $host_a) = ($1, $2, $3, $4);
    }
    else { die "aargh $a\n"; }
    if ($b =~ /^(\D+)(\d+)(\D+)(teeny|bitsy|weeny)$/) {
	($prefix_b, $num_b, $suffix_b, $host_b) = ($1, $2, $3, $4);
    }
    elsif ($b =~ /^(\D+)(teeny|bitsy|weeny)$/) {
	($prefix_b, $host_b) = ($1, $2);
    }
    elsif ($b =~ /^(\D+)(\d+)v(\d+)(teeny|bitsy|weeny)$/) {
	($prefix_b, $num_b, $suffix_b, $host_b) = ($1, $2, $3, $4);
    }
    else { die "bargh $b\n"; }

    return ($prefix_a cmp $prefix_b) if ($prefix_a ne $prefix_b);
    return ($num_a <=> $num_b) if ($num_a ne $num_b);
    return($suffix_a cmp $suffix_b) if ($suffix_a ne $suffix_b);
    return($host_a cmp $host_b);
}

foreach $result  (sort bymine (keys %results)) {
    printf "%s\n", $results{$result};
}

