#!/usr/local/bin/perl -w

sub usage {
    warn(
	 "Usage: pgraph.pl  [<filename>...]+\n");
    exit 1;
}

sub arguments {
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    &usage();
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}

sub processit { # IN_FILEHANDLE 
    local($IN) = shift;

    while(<$IN>) {
	next if (/mismatch/);
	next if (/0 voltage measured/);
	($itsy, $command, $condition, $uid, $Pavg, $Rerr) = 
	    $_ =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+/;
#	print "$itsy, $command, $condition, $uid, $Pavg, $Rerr\n";

	if (($uid eq "batt") || ($uid eq "core")) {

	    if ($Pavg < 0) {
		die "Pavg too big ($Pavg)!\n" if ($Pavg < -.02);
		$Pavg = 0;
	    }

	    $prefix = $command."-".$uid;
	    $tmp = sprintf("$condition#%10.10f#%10.10f", $Pavg, $Rerr);
	    die if $results{$prefix};
	    $results{$prefix} = $tmp;
	}
	elsif ($uid eq "coresup") {
	    ($Eff) = $_ =~ /^itsy.*Eff\s+(\S+)%\s+/;
	    $prefix = $command."-".$uid;
	    $tmp = sprintf("$condition#%10.10f#%10.10f", $Pavg, $Eff);
	    die if $results{$prefix};
	    $results{$prefix} = $tmp;
	}
    }
}

&arguments();

FILE:

if ($#ARGV < 0) {
    &usage();
}

@complist = ("idle", "wav", "tts", "mpeg");
@speeds = ( 59, 74, 88, 103, 118, 132, 147, 162, 177, 192, 206, 221, 236, 265, 295);
@colors = ("1 0 0", "0 1 0", "0 0 1", "1 1 0", ".4 0 .7", ".7 .2 .1", ".7 0 .5", ".2 .5 .7", ".1 .6 .5");
@marks = ("circle", "box", "diamond", "triangle");

sub printgraphheader {
    local($volt) = $_[0];
    local($exp) = $_[1];
    local($speed);
    printf "newgraph\n";
    printf "xaxis size 4 no_auto_hash_marks label : speed\n";
    foreach $speed (@speeds) {
	next if (($exp eq "tts") && ($speed eq "59"));
	next if (($exp eq "mpeg") && ($speed < 192));
	printf "hash_at %d hash_label at %d : %d\n", $speed, $speed, $speed;
    }
}


sub bynumber { return ($a <=> $b); }

while ($#ARGV >= 0) {
    $infile = shift @ARGV;
    open(FILE, $infile) || (die "Can't open $infile: $!\n");
    
    processit(FILE);
    close(FILE);

    foreach $exp (@complist) {

	$colorindex = 0;

	$filename1 = $exp."-power.jgr";
	$filename2 = $exp."-eff.jgr";
	$filename3 = $exp."-power-eff.jgr";
	open(G, "> $filename1") || (die "Can't open $filename1: $!\n");
	open(H, "> $filename2") || (die "Can't open $filename2: $!\n");
	open(I, "> $filename3") || (die "Can't open $filename3: $!\n");

	select(G);
	printgraphheader($volt, $exp);
	printf "yaxis size 3 label : power (mW)\n";
	printf "yaxis min 0\n";
	printf "legend defaults hjr x 295\n";
	select(H);
	printgraphheader($volt, $exp);
	printf "yaxis label : efficiency\n";
	printf "yaxis min 70\n";
	printf "yaxis max 110\n";
	select(I);
	printf "newgraph\n";
	printf "xaxis label : power (mW)\n";
	printf "yaxis label : efficiency\n";
	printf "yaxis min 70\n";
	printf "yaxis max 110\n";
		    
	printf G "title : %s-%s\n", $exp, "batt";
	printf H "title : %s-%s\n", $exp, "efficiency";
	printf I "title : %s-%s\n", $exp, "eff vs core power";

	@junk = split("\#", $results{$exp."295v3-batt"});
	$xmarksize = 6;
	$xmarksize = 3 if ($exp =~ "mpeg");
	$ymarksize = 1.5*$junk[1]*.0185;
	printf G "\nnewcurve fill 1 color 0 0 0 (* %s *) marktype box marksize %d %d \npts\n",
	$exp, $xmarksize, $ymarksize;
	$volt = "v0";
	foreach $speed (@speeds) {
	    $volt = "v1" if ($speed > 88);
	    $volt = "v2" if ($speed > 192);
	    $volt = "v3" if ($speed > 236);
		$uid = "batt";
		$tmp = $results{$exp.$speed.$volt."-".$uid};
		if ($tmp) {
		    @junk = split("\#", $tmp);
		    $pavg = $junk[1];
		    printf G "%d %f\n", $speed, $pavg;
		}
	}


	foreach $volt ("v3", "v2", "v1", "v0") {

	    printf G "\nnewcurve marktype %s cfill %s color %s (* %s %s *) label : %s \npts\n", 
	    $marks[$colorindex], $colors[$colorindex], $colors[$colorindex], $exp, $volt, $volt;
	    printf H "\nnewcurve fill 1 color %s (* %s %s *) label : %s \npts\n", 
	    $colors[$colorindex], $exp, $volt, $volt;
	    printf I "\nnewcurve fill 1 color %s (* %s %s *) label : %s \npts\n", 
	    $colors[$colorindex], $exp, $volt, $volt;

	    %pr = ();

	    foreach $speed (@speeds) {
		
		$pavg = "";

		$uid = "batt";
		$tmp = $results{$exp.$speed.$volt."-".$uid};
		
		if ($tmp) {
		    @junk = split("\#", $tmp);
		    $pavg = $junk[1];

		    printf G "%d %f\n", $speed, $pavg;
		}

		$uid = "coresup";
		$tmp = $results{$exp.$speed.$volt."-".$uid};

		if ($tmp) {
		    @junk = split("\#", $tmp);
		    $eff = $junk[2];

		    printf H "%d %f\n", $speed, $eff;
		}

		$uid = "core";
		$tmp = $results{$exp.$speed.$volt."-".$uid};

		if ($tmp) {
		    @junk = split("\#", $tmp);
		    $pavg = $junk[1];
#		    printf STDERR "setting $exp $speed $volt $uid $pavg $eff\n";
		    die "duplicate value! $pavg $exp $speed $volt $uid" if ($pr{$pavg});
		    $pr{$pavg} = $eff;
		}

	    }
	    
	    foreach $pavg (sort bynumber (keys %pr)) {
		printf I "%f %f\n", $pavg, $pr{$pavg};
	    }

	    $colorindex++;
	}
	close(G);
	close(H);
    }
}


