#!/usr/local/bin/perl -w

sub usage {
    warn(
	 "Usage: prange -data -nbint <number> -tres <number> -nchars <number> -echar <number> \n" .
	 "  -host <{teeny/weeny/bitsy}> [-noprint]\n" .
	 "  [-forcemaxt <seconds>]\n" .
	 "  [-noround]\n" .
	 "  [-pall] [-goofy]\n" .
	 "  [<filename>...]\n");
    exit 1;
}

sub arguments {
    $NbInt = 0;
    $Tres = 0;
    $nchars = 0;
    $Echar = 0;
    $bitsy = $weeny = $teeny = 0;
    $noprint = 0;
    $forcemaxt = 0;
    $noround = 0;
    $pall = 0;
    $goofy = 0;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-nbint", 0, length $_)) {
		$NbInt = shift @ARGV;
	    } elsif ($_ eq substr("-tres", 0, length $_)) {
		$Tres = shift @ARGV;
	    } elsif ($_ eq substr("-nchars", 0, length $_)) {
		$nchars = shift @ARGV;
	    } elsif ($_ eq substr("-echar", 0, length $_)) {
		$Echar = shift @ARGV;
	    } elsif ($_ eq substr("-host", 0, length $_)) {
		local($host) = shift @ARGV;
 		if    ($host eq "weeny") { $weeny = 1; }
		elsif ($host eq "teeny") { $teeny = 1; }
		elsif ($host eq "bitsy") { $bitsy = 1; }
		else { usage(); }
	    } elsif ($_ eq substr("-noprint", 0, length $_)) {
		$noprint = 1;
	    } elsif ($_ eq substr("-forcemaxt", 0, length $_)) {
		$forcemaxt = shift @ARGV;
	    } elsif ($_ eq substr("-noround", 0, length $_)) {
		$noround = 1;
	    } elsif ($_ eq substr("-pall", 0, length $_)) {
		$pall = 1;
	    } elsif ($_ eq substr("-goofy", 0, length $_)) {
		$goofy = 1;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
    &usage() if (($NbInt == 0) || ($Tres == 0) || ($Echar == 0));
    &usage() if (($nchars > 0) && ($noprint));
}

%uids = (
	 "vin", 0,
	 "vrbatt", 0,
	 "vrcorein", 0,
	 "vrmainin", 0,
	 "vcore", 0,
	 "vrcore", 0,
	 "vanalog", 0,
	 "vranalog", 0,
	 "vdigital", 0,
	 "vrdigital", 0,
	 "vrdram", 0);


sub inituid { 
    local($uid) = $_[0];
    $data{$uid} = 0;
}

sub doiter { 
    local($t, $uid, $value)
	= ($_[0], $_[1], $_[2]);

    $uids{$uid} = 1;  # it has been initialized

    $data{$uid} = $value if ($value > $data{$uid});
}


sub printdata {
    local($uid) = $_[0];

    printf "%f\n", $data{$uid};
}

sub initit {
    foreach $uid (keys %uids) {
	inituid($uid);
    }
}

sub processit { # IN_FILEHANDLE 
    local($IN) = $_[0];

    @itmp = split('/', $infile);
    @otmp = split(/\./, $filename);

    ($itsy, $type) = $itmp[$#itmp-1] =~ /^(\d+)(\D.+)$/;
    $itsy = "000" if (! $itsy);
    $type = "" if (! $type);
    $handle = $otmp[0];


    while(<$IN>) {
	next if (/^\#/);
        if ($weeny) {
	    if (/^\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+$/) {
		($Tstart_a, 
		 $Vin_avg_a, $Vin_min_a, $Vin_max_a, $VRbatt_avg_a, $VRbatt_min_a, $VRbatt_max_a, 
		 $VRcorein_avg_a, $VRcorein_min_a, $VRcorein_max_a, 
		 $VRmainin_avg_a, $VRmainin_min_a, $VRmainin_max_a, 
		 $Tend_a) 
		    = split(/ /, $_);
	    }
	    else {
		next;
	    }

	    doiter($Tstart_a, "vin", $Vin_max_a);
	    doiter($Tstart_a, "vrbatt", $VRbatt_max_a);
	    doiter($Tstart_a, "vrcorein", $VRcorein_max_a);
	    doiter($Tstart_a, "vrmainin", $VRmainin_max_a);
	}


        if ($teeny) {
	    if (/^\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+$/) {
		($Tstart_a, 
		 $Vcore_avg_a, $Vcore_min_a, $Vcore_max_a, $VRcore_avg_a, $VRcore_min_a, $VRcore_max_a, 
		 $Vanalog_avg_a, $Vanalog_min_a, $Vanalog_max_a, $VRanalog_avg_a, $VRanalog_min_a, $VRanalog_max_a, 
		 $Tend_a) 
		    = split(/ /, $_);
	    }
	    else {
		next;
	    }

	    doiter($Tstart_a, "vcore", $Vcore_max_a);
	    doiter($Tstart_a, "vrcore", $VRcore_max_a);
	    doiter($Tstart_a, "vanalog", $Vanalog_max_a);
	    doiter($Tstart_a, "vranalog", $VRanalog_max_a);
	}

        if ($bitsy) {
	    if (/^\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+\s\S+$/) {
		($Tstart_a, 
		 $Vdigital_avg_a, $Vdigital_min_a, $Vdigital_max_a, 
		 $VRdigital_avg_a, $VRdigital_min_a, $VRdigital_max_a, 
		 $VRdram_avg_a, $VRdram_min_a, $VRdram_max_a, 
		 $Tend_a) 
		    = split(/ /, $_);
	    }
	    else {
		next;
	    }

	    doiter($Tstart_a, "vdigital", $Vdigital_max_a);
	    doiter($Tstart_a, "vrdigital", $VRdigital_max_a);
	    doiter($Tstart_a, "vrdram", $VRdram_max_a);

	}
    }

}

sub finishit {
    foreach $uid (keys %uids) {
	if ($uids{$uid}) {
	    $tmp = $handle;
	    $tmp .= "-" . $type if $type;
	    $tmp .= ":" . $uid;
	    printf "itsy \#$itsy, %28s ", $tmp;

	    printdata($uid);
	}
    }
}


&arguments();

FILE:

if ($#ARGV < 0) {
    &usage();
}
initit();
while ($#ARGV >= 0) {
    $infile = shift @ARGV;
    open(FILE, $infile) || (die "Can't open $infile: $!\n");
    
    # discard path
    @data = split('/', $infile);
    $filename = $data[$#data];

    if ($infile =~ /\.gz/) {
	close(FILE);
	open(FILE, "gzcat $infile |") || (die "gzcat of $infile failed: $!\n");
    }

    processit(FILE);

    close(FILE);

}
finishit();


if ($pall) {
    open(FILE, "> foo.jgr") || (die "open of foo.jgr failed: $!\n");

    printf FILE "newgraph\n";
    printf FILE "legend defaults hjl vjt\n";
    printf FILE "xaxis size 12\n";
    printf FILE "newcurve color 1 0 0 label : core\n";
    printf FILE "pts\n";
    $i = 0;
    foreach $pt (@corelist) {
	printf FILE "%d %f\n", $i, $pt;
	$i++;
    }

    printf FILE "newcurve color 0 1 0 label : corein\n";
    printf FILE "pts\n";
    $i = 0;
    foreach $pt (@coreinlist) {
	printf FILE "%d %f\n", $i, $pt;
	$i++;
    }

    printf FILE "newcurve color 0 0 1 label : greater?\n";
    printf FILE "pts\n";
    $i = 0;
    foreach $pt (@coreinlist) {
	if ($coreinlist[$i] > $corelist[$i]) {
	    printf FILE "%d %f\n", $i, -.1;
	}
	else {
	    printf FILE "%d %f\n", $i, -.2;
	}
	$i++;
    }
}
