#!/usr/local/bin/perl -w

sub usage {
    warn(
	 "Usage: relative.pl [-round -graph]   [<filename>...]+\n");
    exit 1;
}

%uidlist = (
"spkr", "spkr",
"codec", "codec",
"dram", "dram",
"lcd", "lcd",
"main", "main",
"core", "core",
"mon", "mon",
"mainsup", "main sup",
"coresup", "core sup",
);

sub arguments {
    $dontround = 1;
    $graph = 0;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-round", 0, length $_)) {
		$dontround = 0;
	    } elsif ($_ eq substr("-graph", 0, length $_)) {
		$graph = 1;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}

sub pline {
    local($itsy) = shift;
    local($tmp) = shift;
    local($Pavg) = shift;
    local($Rerr) = shift;
    local($Percent) = shift;
    printf "itsy \#$itsy, %28s ", $tmp;
    printf 
	"Pavg %.${p}f mW, RErr %.${p}f%%    Percent %.${p}f%%\n", $Pavg, $Rerr, $Percent;
}

sub generate_new_data {
    local($command) = shift;
    local($condition) = shift;
    local($sum) = shift;
    local($uid, $Pavg, $Rerr);
    local($tmp, $datum, $prefix);

    foreach $datum (@data) {
	($uid, $Pavg, $Rerr) = split(/\#/, $datum);
#	die "Don't add up! batt $Pavg added to $sum\n" if (($uid eq "batt") && (abs($Pavg-$sum)>.02));
	next if ($uid eq "batt");
	if (! $graph) {
	    $tmp = $command . "-" . $condition . ":" . $uid;
	    pline($itsy, $tmp, $Pavg, $Rerr, 100*$Pavg/$sum);
	}
	else {
	    if ($sum > 0) {
		$prefix = $command."-".$uid;
		$tmp = sprintf("$condition#%10.10f#%10.10f#%10.10f#%10.10f", $Pavg, $Rerr,100*$Pavg/$sum, $sum);
		die if $results{$prefix};
		$results{$prefix} = $tmp;
	    }
	}
    }
}

sub processit { # IN_FILEHANDLE 
    local($IN) = shift;

    $p = 6;
    $p = 3 if (!$dontround);

    $currentcommand = "";
    $currentsum = 0;
    @data = ();
    while(<$IN>) {
	next if (/mismatch/);
	next if (/0 voltage measured/);
	($itsy, $command, $condition, $uid, $Pavg, $Rerr) = 
	    $_ =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+/;
#	print "$itsy, $command, $condition, $uid, $Pavg, $Rerr\n";

	next if (($uid eq "corein") || ($uid eq "mainin") || ($uid eq "digital") || ($uid eq "analog"));
	next if (! $uidlist{$uid});

	if ($currentcommand ne $command) {
	    generate_new_data($currentcommand, $condition, $currentsum);

	    $currentcommand = $command;
	    $currentsum = 0;
	    @data = ();
	}

	if ($Pavg < 0) {
	    die "Pavg too big ($Pavg)!\n" if ($Pavg < -.02);
	    $Pavg = 0;
	}
	if ($uid ne "batt") {
	    $currentsum += $Pavg;
	}
	push(@data, $uid . "\#" . $Pavg . "\#" . $Rerr);

    }
    generate_new_data($currentcommand, $condition, $currentsum);
}

&arguments();

FILE:

if ($#ARGV < 0) {
    &usage();
}

@complist = ("idle", "wav", "tts");
@speeds = ( 59, 74, 88, 103, 118, 133, 147, 162, 177, 192, 206);
@colors = ("1 0 0", "0 1 0", "0 0 1", ".5 .5 0", ".4 0 .7", ".7 .2 .1", ".7 0 .5", ".2 .5 .7", ".1 .6 .5");

sub printgraphheader {
    local($volt) = $_[0];
    local($exp) = $_[1];
    local($speed);
    printf "newgraph\n";
    printf "xaxis no_auto_hash_marks label : speed\n";
    foreach $speed (@speeds) {
	next if (($exp eq "tts") && ($speed eq "59"));
	last if (($speed > 88) && ($volt eq "low"));
	printf "hash_at %d hash_label at %d : %d\n", $speed, $speed, $speed;
    }
    printf "yaxis min 0\n";
}


@uids = (
"dram",
"core",
"codec",
"spkr",
"lcd",
"main",
"mon",
"mainsup",
"coresup",
);


while ($#ARGV >= 0) {
    $infile = shift @ARGV;
    open(FILE, $infile) || (die "Can't open $infile: $!\n");
    
    processit(FILE);
    close(FILE);

    if ($graph) {
	foreach $type ("rel", "abs") {
	    foreach $exp (@complist) {
		foreach $volt ("high", "low") {

		    $colorindex = 0;

		    $filename = $exp."-".$volt."-".$type."breakdown.jgr";
		    open(G, "> $filename") || (die "Can't open $filename: $!\n");
		    select(G);
		    $firstcomm = "";

		    printgraphheader($volt, $exp);
		    printf "yaxis max 450\n" if ($type eq "abs");
		    
		    foreach $speed (@speeds) {
			$colorindex = 0;
			$currentheight = 100 if ($type eq "rel");
			$firstspeed = 1;

			foreach $uid (@uids) {
			    $tmp = $results{$exp.$speed.$volt."-".$uid};
			    if ($tmp) {
				@junk = split("\#", $tmp);

				if ($firstspeed) {
				    $currentheight = $junk[4] if ($type eq "abs");
				    $firstspeed = 0;
				}

				if ((! $firstcomm) || ($firstcomm eq $exp.$speed.$volt)) {
				    if (! $firstcomm) {
					printf "title : %s-%s\n", $exp, $volt;
					$firstcomm = $exp.$speed.$volt;
				    }
				    printf "newcurve marktype xbar fill 1 color %s (* %s%s%s *) label : %s \npts\n", 
				    $colors[$colorindex], $exp, $speed, $volt, $uid;
				}
				else {
				    printf "newcurve marktype xbar fill 1 color %s (* %s %s%s%s *) \npts\n", 
				    $colors[$colorindex], $uid, $exp, $speed, $volt;
				}
				$colorindex++;
				printf "%d %f\n", $speed, $currentheight;
				if ($type eq "rel") {
				    $currentheight -= $junk[3];
				}
				else {
				    $currentheight -= $junk[1];
				}
			    }
			}
		    }
		}
		close(G);
	    }
	}
    }
}


