#!/usr/local/bin/perl -w

sub usage {
    warn(
	 "Usage: relpaper.pl [-round -graph]   [<filename>...]+\n");
    exit 1;
}

sub arguments {
    $dontround = 1;
    $graph = 0;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-round", 0, length $_)) {
		$dontround = 0;
	    } elsif ($_ eq substr("-graph", 0, length $_)) {
		$graph = 1;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}

sub pline {
    local($itsy) = shift;
    local($tmp) = shift;
    local($Pavg) = shift;
    local($Rerr) = shift;
    local($Percent) = shift;
    local($minPercent) = shift;
    local($maxPercent) = shift;
    printf "$itsy: %25s ", $tmp;
    $a = sprintf("%4.4f", $Pavg);
    if (($Rerr > 999) || ($Pavg <= .005)) {
	printf "PB %8s Pavg %6s\n", sprintf("%3.4f",$Pavgbatt), "-";
    }
    else {
	$b = sprintf("%3.4f", $Rerr);
	$c = sprintf("%2.1f", $Percent);
	$d = sprintf("%2.1f", $Percent-$minPercent);
	$e = sprintf("%2.1f", $maxPercent-$Percent);
	$f = $c;
	if ((($Percent-$minPercent) > 1) || (($maxPercent-$Percent)>1)) {
	    $f = sprintf("%2.0f  ", $Percent);
	}
	printf "PB %8s Pavg %8s mW, RErr %8s%%  Percents  %4s  %4s %4s   %4s\n", sprintf("%3.4f",$Pavgbatt), $a, $b, $d, $c, $e, $f;
	$psum += $f;
	$ppsum += $c;
    }
#    printf "Pavg %.${p}f mW, RErr %.${p}f%%    Percent %.${p}f%%\n", $Pavg, $Rerr, $Percent;
}

sub generate_new_data {
    local($command) = shift;
    local($condition) = shift;
    local($sum) = shift;
    local($uid, $Pavg, $Rerr);
    local($tmp, $datum, $prefix);
    local($Pavgbatt, $Rerrbatt, $Pavgbattmin, $Pavgbattmax);

    $psum = 0;
    $ppsum = 0;
    $datum = $data{"batt"};
    ($uid, $Pavgbatt, $Rerrbatt) = split(/\#/, $datum);
    $Pavgbattmin = $Pavgbatt-($Pavgbatt*$Rerrbatt/100);
    $Pavgbattmax = $Pavgbatt+($Pavgbatt*$Rerrbatt/100);

    foreach $u (@uidlist) {
	$datum = $data{$u};
	($uid, $Pavg, $Rerr) = split(/\#/, $datum);
	die "Weird shit!\n" if (($uid eq "batt") && ($Pavg != $Pavgbatt));
	die "Don't add up! batt $Pavg added to $sum\n" if (($uid eq "batt") && (abs($Pavg-$sum)>0.000001));
	die "Don't add up! batt $Pavg added to $sum\n" if (($uid eq "batt") && (abs($Pavg-$sum)>.02));
	next if ($uid eq "batt");
	if (! $graph) {
	    $tmp = $command . "-" . $condition . ":" . $uid;
	    pline($itsy, $tmp, $Pavg, $Rerr, 
		  100*$Pavg/$Pavgbatt, 
		  100*($Pavg-($Pavg*$Rerr/100))/$Pavgbattmax,
		  100*($Pavg+($Pavg*$Rerr/100))/$Pavgbattmin);
	}
	else {
	    $prefix = $command."-".$uid;
	    $tmp = sprintf("$condition#%10.10f#%10.10f#%10.10f#%10.10f", $Pavg, $Rerr,100*$Pavg/$Pavgbatt, $Pavgbatt);
	    die if $results{$prefix};
	    $results{$prefix} = $tmp;
	}
    }
    if (! $graph) {
	printf "                                                                                                          %3.1f        %3.1f\n\n", $ppsum, $psum;
    }
}

sub processit { # IN_FILEHANDLE 
    local($IN) = shift;

    $p = 6;
    $p = 2 if (!$dontround);

    $currentcommand = "";
    $currentsum = 0;
    @uidlist = ();
    %data = ();
    while(<$IN>) {
	next if (/mismatch/);
	next if (/0 voltage measured/);
	($itsy, $command, $condition, $uid, $Pavg, $Rerr) = 
	    $_ =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+/;
#	print "$itsy, $command, $condition, $uid, $Pavg, $Rerr\n";

	next if (($uid eq "corein") || ($uid eq "mainin") || ($uid eq "digital") || ($uid eq "analog"));

	if ((!$currentcommand) || ($currentcommand ne $command)) {
	    generate_new_data($currentcommand, $condition, $currentsum) if ($currentcommand);

	    $currentcommand = $command;
	    $currentsum = 0;
	    @uidlist = ();
	    %data = ();
	}

#	if ($Pavg < 0) {
	    die "Pavg too big ($Pavg)!\n" if ($Pavg < -.02);
#	    $Pavg = 0;
#	}
	if ($uid ne "batt") {
	    $currentsum += $Pavg;
	}
	push(@uidlist, $uid);
	$data{$uid} = $uid . "\#" . $Pavg . "\#" . $Rerr;
    }
    generate_new_data($currentcommand, $condition, $currentsum);
}

%expmap = (
	   "deepsleep",    "Deep\\\nsleep",
	   "sleep",        "Sleep",
	   "sleeplcd",     "Sleep\\\nLCD",
	   "idle",         "Idle",
	   "wav",          "WAV",
	   "tts",          "DECtalk",
	   "mpeg",         "MPEG-1"
	   );
%speedmap = (
	   "59",    "59",
	   "74",    "74",
	   "88",    "88",
	   "133",   "133",
	   "206",   "206",
);

# dict?  sleeplcd?
@exps = ("break-26", "deepsleep", "sleep", "sleeplcd", "break-104", "idle", "break-831", "wav", "tts", "mpeg");
@suffixes = ("59low", "88low", "59high", "88high", "133high", "206high");
@colors = ("fill 1 color 1 0 0", 
	   "fill 1 color 0 1 0", 
	   "fill 1 color 0 0 1", 
	   "fill 1 color .5 .5 0", 
	   "fill 1 color .4 0 .7",
	   "fill 1 color .7 .2 .1",
	   "fill 1 color .7 0 .5",
	   "fill 1 color .2 .5 .7",
	   "fill 1 color .1 .6 .5",
	   );

@colors = ("fill 0.75 pattern solid",
	   "fill 0.25 pattern stripe 45",
	   "fill 0.75 pattern stripe -45",
	   "fill 1.0  pattern solid",
	   "fill 0.75 pattern stripe 45",
	   "fill 0.0  pattern solid",
	   "fill 0.5  pattern stripe -45",
	   "fill 0.5  pattern solid",
	   "fill 0.5  pattern stripe 45",
	   );
	   
sub printgraphheader {
    local(*exps) = $_[0];
    local($exp, $count, $suffix, $seen);
    $count = 0;
    printf "newgraph\n";
    printf "legend defaults x 39 y 25 hjl vjt\n";
    printf "xaxis hash_labels fontsize 8 size 6.75\n";
    printf "yaxis hash_labels fontsize 8 size 3.375\n";
    printf "xaxis no_draw_hash_marks no_auto_hash_labels ";
    foreach $exp (@exps) {
	if ($exp =~ "break") {
	    $count += 1 if ($count > 0);
	    $count++;
	    next;
	}
	$seen = 0;
	foreach $suf (@suffixes) {

	    $suffix = $suf;
	    $suffix = "74low" if (($exp eq "tts") && ($suf eq "59low"));
	    $suffix = "74high" if (($exp eq "tts") && ($suf eq "59high"));

	    if ($results{$exp.$suf."-core"}) {
		$seen = 1;
		$count += 2;
	    }
	    $count += 1 if ((!($exp =~ /mpeg/)) && ($suffix =~ /88low/));
#	    printf STDERR "%d: %s%s\n", $count, $exp, $suf;
	}
	if (!$seen) {
	    $count += 2;
	}
	$count += 2;
#	printf STDERR "%d: %s\n", $count, $exp;
    }
    printf "min 0 max %d\n", $count;
    printf "yaxis min 0\n";
    printf "newstring vjc x 2 y 25 : mW\n";

}


@uids = (
"mon",
"mainsup",
"spkr",
"codec",
"dram",
"lcd",
"main",
"coresup",
"core",
);

%uidmap = (
"mon", "mon",
"mainsup", "main sup",
"spkr", "spkr",
"codec", "codec",
"dram", "DRAM",
"lcd", "LCD",
"main", "main",
"coresup", "core sup",
"core", "core",
);


sub dograph {
    local(*exps) = $_[0];

    $filename = "breakdown.jgr";
    open(G, "> $filename") || (die "Can't open $filename: $!\n");
    select(G);
    
    printgraphheader(*exps);
    $xindex = 0;
    $b = 0;
    
    foreach $exp (@exps) {
	@thissuffixlist = "";
	@thissuffixlist = @suffixes if (! ($exp =~ /sleep/));

	if ($exp =~ /^break-(\d+)$/) {
	    if ($xindex > 0) {
		printf "\ncopygraph\n";
		printf "xaxis no_draw_axis no_draw_axis_label no_draw_hash_marks no_auto_hash_labels\n";
		printf "yaxis min 0 max %d draw_at %d no_auto_hash_labels\n\n", $1, $xindex+1;
		if (abs($1 - 100)<10) {
		    printf "yaxis hash_label at %d : %d\n",  20,  20;
		    printf "yaxis hash_label at %d : %d\n",  40,  40;
		    printf "yaxis hash_label at %d : %d\n",  60,  60;
		    printf "yaxis hash_label at %d : %d\n",  80,  80;
		    printf "yaxis hash_label at %d : %s\n", 100, "100";
		    printf "newstring vjc x %d y 100 : mW\n", $xindex+3;
		}
		elsif ($1 >= 800) {
		    printf "yaxis hash_label at %d : %d\n", 200, 200;
		    printf "yaxis hash_label at %d : %d\n", 400, 400;
		    printf "yaxis hash_label at %d : %d\n", 600, 600;
		    printf "yaxis hash_label at %d : %s\n", 800, "800";
		    printf "newstring vjc x %d y 800 : mW\n", $xindex+3;
		}
		$xindex+=1;
	    }
	    $xindex+=2;
	    next;
	}
	
	foreach $suf (@thissuffixlist) {
	    
	    $colorindex = 0;
	    $firstonbar = 1;

	    foreach $uid (@uids) {
		
		$suffix = $suf;
		$suffix = "74low" if (($exp eq "tts") && ($suf eq "59low"));
		$suffix = "74high" if (($exp eq "tts") && ($suf eq "59high"));

		$tmp = $results{$exp.$suffix."-".$uid};
		
		if ($tmp) {
		    @junk = split("\#", $tmp);
		    
		    if ($firstonbar) {
			if ($suffix =~ /^(\d+)(high|low)$/) {
			    $speed = $1;
			    if ($speed =~ /133/) {
				printf "xaxis hash_label at %f : \\\n\\\n%s\n", $xindex-4, $expmap{$exp};
				printf "xaxis hash_label at %f : \\\n%s\n", $xindex-1, "1.5 V";
				$b++;
			    }
			    elsif ($suffix =~ /88low/) {
				printf "xaxis hash_label at %f : \\\n%s\n", $xindex-1, "1.25 V";
				$b++;
			    }
			    elsif ($exp =~ /mpeg/) {
				printf "xaxis hash_label at %d : \\\n%s\\\n%s\n", $xindex, "1.5 V", $expmap{$exp};
			    }
			    printf "xaxis hash_label at %d : %s\n", $xindex, $speedmap{$speed};
			}
			else {
			    printf "xaxis hash_label at %d : %s\n", $xindex, $expmap{$exp};
			}

			$currentheight = $junk[4];
			$firstonbar = 0;
		    }
		    
		    if ((! $firstcomm) || ($firstcomm eq $exp.$suffix)) {
			$firstcomm = $exp.$suffix;
			printf "newcurve marktype xbar marksize 1.8 1 %s (* %s%s *) label : %s \npts\n", 
			$colors[$colorindex], $exp, $suffix, $uidmap{$uid};
		    }
		    else {
			printf "newcurve marktype xbar marksize 1.8 1 %s (* %s %s%s *) \npts\n", 
			$colors[$colorindex], $uid, $exp, $suffix;
		    }
		    $colorindex++;
		    printf "%d %f\n", $xindex, $currentheight;
		    $currentheight -= $junk[1];
		}
		
	    }
	    $xindex += 1 if ((! $firstonbar) && ($suffix =~ /88low/));
	    $xindex += 2 if (! $firstonbar);
	}
	$xindex += 2;
    }
    close(G);
}
    
    
&arguments();

FILE:

if ($#ARGV < 0) {
    &usage();
}

while ($#ARGV >= 0) {
    $infile = shift @ARGV;
    open(FILE, $infile) || (die "Can't open $infile: $!\n");
    
    processit(FILE);
    close(FILE);

    if ($graph) {

	$firstcomm = "";  # only once!

	dograph(*exps);
    }
}
