#!/usr/local/bin/perl -w

sub usage {
    warn(
	 "Usage: generate.pl -dontround [<filename>...]+\n");
    exit 1;
}

sub arguments {
    $dontround = 1;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-dontround", 0, length $_)) {
		$dontround = 1;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}

sub pline {
    local($Pavg) = shift;
    local($Rerr) = shift;
    local($Eff) = shift;
    printf 
	"Pavg %.${p}f mW, RErr %.${p}f%%    Eff %.${p}f%%\n", 
	$Pavg, $Rerr, $Eff*100;
}

sub generate_new_data {
    local($command) = shift;
    local($Pavg, $Rerr);
    local($prefix) = $command . "-" . $condition;
    local($tmp);

    $Pavg = $data{"coreinPavg"}-$data{"corePavg"};
    $Rerr = ($data{"coreinRerr"}*abs($data{"coreinPavg"})+
	     $data{"coreRerr"}*abs($data{"corePavg"}))
	/ abs($Pavg);
    $tmp = $prefix . ":" . "coresup";
    printf "itsy \#$itsy, %28s ", $tmp;
    pline($Pavg, $Rerr, $data{"corePavg"}/$data{"coreinPavg"});

    $Pavg = $data{"maininPavg"}-$data{"mainPavg"}-$data{"analogPavg"}-$data{"dramPavg"};
    $Rerr = ($data{"maininRerr"}*abs($data{"maininPavg"})+
	     $data{"mainRerr"}*abs($data{"mainPavg"})+
	     $data{"analogRerr"}*abs($data{"analogPavg"})+
	     $data{"dramRerr"}*abs($data{"dramPavg"}))
	/ abs($Pavg);
    $tmp = $prefix . ":" . "mainsup";
    printf "itsy \#$itsy, %28s ", $tmp;
    pline($Pavg, $Rerr, ($data{"mainPavg"}+$data{"analogPavg"}+$data{"dramPavg"})/$data{"maininPavg"});

    $Pavg = $data{"battPavg"}-$data{"maininPavg"}-$data{"coreinPavg"};
    $Rerr = ($data{"battRerr"}*abs($data{"battPavg"})+
	     $data{"maininRerr"}*abs($data{"maininPavg"})+
	     $data{"coreinRerr"}*abs($data{"coreinPavg"}))
	/ abs($Pavg);
    
    $tmp = $prefix . ":" . "mon";
    printf "itsy \#$itsy, %28s ", $tmp;
    pline($Pavg, $Rerr, -1/100);


}

%data = ();
%uids = ("batt", 0,
	 "mainin", 0,
	 "corein", 0,
	 "core", 0,
	 "main", 0,
	 "dram", 0,
	 "analog", 0);

sub processit { # IN_FILEHANDLE 
    local($IN) = $_[0];

    $p = 6;
    $p = 3 if (!$dontround);

    $currentcommand = "";
    while(<$IN>) {
	next if (/mismatch/);
	next if (/0 voltage measured/);
	($itsy, $command, $condition, $uid, $Pavg, $Rerr, $Plowbnd, $Pupbnd, $Tres) = 
	    $_ =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+low:(\S+)\s+up:(\S+)\s+Tres:(\S+)\s+/;
#	print "$itsy, $command, $condition, $uid, $Pavg, $Rerr, $Plowbnd, $Pupbnd, $Tres\n";

	if ($currentcommand ne $command) {
	    local($uid);
	    if ($currentcommand) {
		generate_new_data($currentcommand);
	    }
	    foreach $uid (keys %uids) {
		$data{$uid."Pavg"}=-100000000;
		$data{$uid."Rerr"}=1000000000;
	    }
	    $currentcommand = $command;
	}

	$tmp = $command;
	$tmp .= "-" . $condition;
	$tmp .= ":" . $uid;
	printf "itsy \#$itsy, %28s ", $tmp;
	printf 
	    "Pavg %.${p}f mW, RErr %.${p}f%%    low:%.${p}f up:%.${p}f Tres:%d\n", 
	    $Pavg, $Rerr,
	    $Plowbnd, $Pupbnd, 
	    $Tres;

	$data{$uid."Pavg"}=$Pavg;
	$data{$uid."Rerr"}=$Rerr;

    }
    generate_new_data($currentcommand) if ($currentcommand);
}

&arguments();

FILE:

if ($#ARGV < 0) {
    &usage();
}

while ($#ARGV >= 0) {
    $infile = shift @ARGV;
    open(FILE, $infile) || (die "Can't open $infile: $!\n");
    
    processit(FILE);
    close(FILE);
}
