#!/usr/local/bin/perl -w

sub usage {
    warn(
	 "Usage: mtable.pl -rawtables -perrors  [<filename>...]+\n");
    exit 1;
}

sub arguments {
    $rawtables = 0;
    $perrors = 0;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-rawtables", 0, length $_)) {
		$rawtables = 1;
	    } elsif ($_ eq substr("-perrors", 0, length $_)) {
		$perrors = 1;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}


sub processit { # IN_FILEHANDLE 
    local($IN) = shift;

    %data = ();
    %adata = ();
    while(<$IN>) {
	if (/^(\d+):\s+(\S+)-\S+:(\S+)\s*PB\s+(\S+)\s+Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+.*\s(\S+)\s*$/) {
	    ($itsy, $command, $uid, $Pbatt, $Pavg, $Rerr, $Percent) = ($1, $2, $3, $4, $5, $6, $7);
#	print "$itsy, $command, $uid, $Pbatt, $Pavg, $Rerr, $Percent\n";

	    $data{$command} = $Pbatt;
	    $adata{$command."-".$uid} = $Pavg;
	    $data{$command."-".$uid} = $Percent;
	    $rdata{$command."-".$uid} = $Rerr;
	}
	elsif (/^(\d+):\s+(\S+)-\S+:(\S+)\s*PB\s+(\S+)\s+Pavg\s+-\s*$/) {
	    ($itsy, $command, $uid, $Pbatt) = ($1, $2, $3, $4);
#	print "$itsy, $command, $uid, $Pbatt, $Pavg, $Rerr, $Percent\n";

	    $data{$command} = $Pbatt;
	    $adata{$command."-".$uid} = "-";
	    $data{$command."-".$uid} = "-";
	    $rdata{$command."-".$uid} = "-";
	}
	
    }
}

sub processitraw { # IN_FILEHANDLE 
    local($IN) = $_[0];

    %data = ();
    %adata = ();
    while(<$IN>) {
	next if (/mismatch/);
	next if (/0 voltage measured/);
	($itsy, $command, $condition, $uid, $Pavg, $Rerr, $Plowbnd, $Pupbnd, $Tres) = 
	    $_ =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+low:(\S+)\s+up:(\S+)\s+Tres:(\S+)\s+/;
#	print "$itsy, $command, $condition, $uid, $Pavg, $Rerr, $Plowbnd, $Pupbnd, $Tres\n";

	if (($Rerr > 999) || ($Pavg <= .005)) {
	    $Pavg="-";
	    $Rerr="-";
	}

	$data{$command}=$condition;
	$data{$command."-".$uid}=$Pavg;
	$rdata{$command."-".$uid}=$Rerr;

    }
}

%expmap = (
	   "deepsleep",    "Deep sleep",
	   "sleep",        "Sleep",
	   "sleeplcd",     "Sleep, LCD",
	   "idle",         "Idle",
	   "wav",          "WAV",
	   "tts",          "DECtalk",
	   "mpeg",         "MPEG-1"
	   );
%speedmap = (
	   "59",    "59",
	   "74",    "74",
	   "88",    "88",
           "103",   "103",
           "118",   "118",
	   "133",   "133",
	   "147",   "147",
	   "162",   "162",
	   "177",   "177",
	   "192",   "192",
	   "206",   "206",
);
%voltmap = (
	   "low",    "1.25          ",
	   "high",   "1.5\\phantom{0}",
);
@speeds = ( 59, 74, 88, 103, 118, 133, 147, 162, 177, 192, 206);

@exps = ("deepsleep", "sleep", "sleeplcd", "idle", "wav", "tts", "mpeg");
%keep = ("",      1,
	 "59low", 1,
	 "88low", 1,
	 "59high", 1,
	 "88high", 1,
	 "133high",1,
	 "206high", 1);

@uids = (
"mon",
"mainsup",
"spkr",
"codec",
"dram",
"lcd",
"main",
"coresup",
"core",
);

@rawuids = (
"batt",
"mainin",
"analog",
"dram",
"digital",
"corein",
"core",
);

sub pround {
    local($v) = shift;
    if ($v <= 9.95) {
	printf "  %1.2f           ", $v;
    }
    elsif ($v <= 99.95) {
	printf " %2.1f\\phantom{0} ", $v;
    }
    else {
	printf "%3.0f\\phantom{.00} ", $v;
    }
}

sub prounderror {
    local($v) = shift;
    if ($v <= 99.9) {
	printf "  %2.1f           ", $v+.0499999999999999;
    }
    else {
	printf "%3.0f\\phantom{.0} ", $v+.499999999999999;
    }
}

sub dotables {
    local(*exps) = $_[0];

    $filename = "relbrkdwn";
    $filename .= "2" if ($perrors);
    open(G, "> $filename.tex") || (die "Can't open $filename: $!\n");
    $filename = "absbrkdwn";
    $filename .= "2" if ($perrors);
    open(H, "> $filename.tex") || (die "Can't open $filename: $!\n");
    
    foreach $exp (@exps) {

	foreach $volt ("low", "high") {

	    foreach $speed (@speeds) {

		$thissuffix = "";
		$thissuffix = $speed.$volt if (! ($exp =~ /sleep/));
		$command = $exp.$thissuffix;

		next if (! $data{$command});

		if ((! $keep{$thissuffix}) && 
		    (! (($exp =~ /tts/) && ($speed =~ 74)))) {
		    printf G "%% ";
		    printf H "%% ";
		}
		else {
		    printf G "  ";
		    printf H "  ";
		}
		printf G "%-14s& ", $expmap{$exp};
		printf H "%-14s& ", $expmap{$exp};
		if ($exp =~ /sleep/) {
		    printf G "\\Ma & \\Mb            & ";
		    printf H "\\Ma & \\Mb            & ";
		}
		else {
		    printf G "%3d & %s & ", $speedmap{$speed}, $voltmap{$volt};
		    printf H "%3d & %s & ", $speedmap{$speed}, $voltmap{$volt};
		}
		
		if (! $perrors) {
		    $Pbatt = $data{$command};
		    if ($exp =~ /deepsleep/) {
			printf STDERR "Warning, futzing round on deepsleep pbatt from 3.9 to 4.0\n";
			$Pbatt = 4.40 if ($Pbatt == 4.395);
		    }
		    select G;
		    pround($Pbatt);
		    printf "& ";
		    select H;
		    pround($Pbatt);
		    printf "& ";
		}

		foreach $uid (@uids) {
		    $Pavg = $data{$command."-".$uid};
		    $Rerr = $rdata{$command."-".$uid};
		    if ($Pavg =~ /-/) {
			printf G "\\Ma";
		    }
		    else {
			printf G "%s", $Pavg;
			if (!($Pavg =~ /\./)) {
			    printf G "\\phantom{.0}";
			}
		    }

		    $aPavg = $adata{$command."-".$uid};
		    if ($aPavg =~ /-/) {
			printf H "\\Ma";
		    }
		    else {
			select H;
			pround($aPavg);
		    }

		    if ($perrors) {
			printf " & ";
			if ($Rerr =~ /-/) {
			    printf G "\\Ma";
			    printf H "\\Ma";
			}
			else {
			    select G;
			    prounderror($Rerr);
			    select H;
			    prounderror($Rerr);
			}
		    }

		    if ($uid ne "core") {
			printf G " & ";
			printf H " & ";
		    }
		    else {
			printf G "\\\\";
			printf H "\\\\";
		    }

		}
		printf G "\n";
		printf H "\n";
		last if ($exp =~ /sleep/);
	    }
	    last if ($exp =~ /sleep/);
	}
    }
}

sub dorawtables {
    local(*exps) = $_[0];

    foreach $exp (@exps) {

	foreach $volt ("low", "high") {

	    foreach $speed (@speeds) {

		$thissuffix = "";
		$thissuffix = $speed.$volt if (! ($exp =~ /sleep/));
		$command = $exp.$thissuffix;

		next if (! $data{$command});

		if ((! $keep{$thissuffix}) && 
		    (! (($exp =~ /tts/) && ($speed =~ 74)))) {
		    printf "%% ";
		}
		else {
		    printf "  ";
		}
		printf "%-14s& ", $expmap{$exp};
		if ($exp =~ /sleep/) {
		    printf "\\Ma & \\Mb            & ";
		}
		else {
		    printf "%3d & %s & ", $speedmap{$speed}, $voltmap{$volt};
		}
		
		$condition = $data{$command};

		foreach $uid (@rawuids) {
		    $Pavg = $data{$command."-".$uid};
		    $Rerr = $rdata{$command."-".$uid};
		    if ($Pavg =~ /-/) {
			printf "\\Ma";
		    }
		    else {
			pround($Pavg);
		    }
		    if ($perrors) {
			printf "& ";
			if ($Rerr =~ /-/) {
			    printf "\\Ma";
			}
			else {
			    prounderror($Rerr);
			}
		    }

		    if ($uid ne "core") {
			printf "& ";
		    }
		    else {
			printf "\\\\";
		    }

		}
		printf "\n";
		last if ($exp =~ /sleep/);
	    }
	    last if ($exp =~ /sleep/);
	}
    }
}

&arguments();

FILE:

if ($#ARGV < 0) {
    &usage();
}

while ($#ARGV >= 0) {
    $infile = shift @ARGV;
    open(FILE, $infile) || (die "Can't open $infile: $!\n");
    
    if ($rawtables) {
	processitraw(FILE);
	dorawtables(*exps);
    }
    else {
	processit(FILE);
	dotables(*exps);
    }
    close(FILE);

}
