#!/usr/local/bin/perl -w

# e.g.
#../bin-full/subtract.pl -part analog spkr codec 34good/out-more 34gnospeaker/out-more >! 34good/out-mm
#../bin-full/subtract.pl -part main lcd much 34good/out-mm 34gnolcd/out-more  >! 34good/out-mmm

sub usage {
    warn(
	 "Usage: subtract.pl -dontround [-particular <orig> <sub> <rem>]   [<filename>...]+\n");
    exit 1;
}

sub arguments {
    $dontround = 1;
    $particular = 0;
    while ($#ARGV >= 0) {
	$_ = shift @ARGV;
	if (/^-/) {
	    if ($_ eq substr("-dontround", 0, length $_)) {
		$dontround = 1;
	    } elsif ($_ eq substr("-particular", 0, length $_)) {
		$particular = 1;
		$orig_uid = shift @ARGV;
		$sub_uid = shift @ARGV;
		$rem_uid = shift @ARGV;
	    } else {
		&usage();
	    }
	}
	else {
	    unshift @ARGV,$_;
	    last;
	}
    }
}

%ignores = ();

sub pline {
    local($itsy) = shift;
    local($tmp) = shift;
    local($Pavg) = shift;
    local($Rerr) = shift;
    printf "itsy \#$itsy, %28s ", $tmp;
    printf 
	"Pavg %.${p}f mW, RErr %.${p}f%%\n", $Pavg, $Rerr;
}

sub calcrerr {
    local($tmp) = shift;
    local($Pavg1) = shift;
    local($Rerr1) = shift;
    local($Pavg2) = shift;
    local($Rerr2) = shift;
    local($PavgNew) = shift;
    local($RerrNew);
    if ($Pavg1+$Pavg2 != 0) {
	$RerrNew = ($Rerr1*abs($Pavg1)+$Rerr2*abs($Pavg2))/abs($PavgNew);
    }
    else {
	$RerrNew = 1000000000;
	printf STDERR "Warning: $tmp Rerr faked due to zero power\n";
    }
    $RerrNew;	
}

sub processit { # IN_FILEHANDLE 
    local($IN1) = shift;
    local($IN2) = shift;

    $p = 6;
    $p = 3 if (!$dontround);

    while(<$IN1>) {
	next if (/mismatch/);
	next if (/0 voltage measured/);
	next if (/dict/);
	($itsy, $command, $condition, $uid, $Pavg, $Rerr) = 
	    $_ =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+/;
#	print "$itsy, $command, $condition, $uid, $Pavg, $Rerr\n";
	if (($uid eq "spkr") || ($uid eq "codec") || ($uid eq "lcd") || ($uid eq "much")) {
	    if (! $ignores{$uid}) {
		printf STDERR "Warning, passing through \"$uid\" directly.\n";
		$ignores{$uid} = 1;
	    }
	    $tmp = $command;
	    $tmp .= "-" . $condition;
	    $tmp .= ":" . $uid;
	    pline($itsy, $tmp, $Pavg, $Rerr);
	    next;
	}

	while (1) {
	    $command2 = "";
	    $in = <$IN2>;
	    last if (!$in);
	    next if ($in =~ (/mismatch/));
	    next if ($in =~ (/0 voltage measured/));
	    ($itsy2, $command2, $condition2, $uid2, $Pavg2, $Rerr2) = 
		$in =~ /^itsy \#(\d+),\s+(\S+)-(\S+):(\S+)\s*Pavg\s+(\S+)\s+mW,\s+R[eE]rr\s+(\S+)%\s+/;
	    next if ($command2 ne $command);
	    last if ($command eq $command2);
	}

#	print "$itsy2, $command2, $condition2, $uid2, $Pavg2, $Rerr2\n";

	if (($itsy ne $itsy2) || ($uid ne $uid2)) {
	    die "Mismatch in input files: $itsy ne $itsy2 OR $uid ne $uid2!\n";
	}

	if (! $particular) {
	    $tmp = $command;
	    $tmp .= "-" . $condition . "_minus_" . $condition2;
	    $tmp .= ":" . $uid;
	    $PavgNew = $Pavg-$Pavg2;
	    $RerrNew = calcrerr($tmp, $Pavg, $Rerr, $Pavg2, $Rerr2, $PavgNew);
	    pline($itsy, $tmp, $PavgNew, $RerrNew);
	}
	else {
	    $tmp = $command;
	    $tmp .= "-" . $condition;
	    $tmp .= ":" . $uid;
	    pline($itsy, $tmp, $Pavg, $Rerr);

	    if ($uid eq $orig_uid) {
		$tmp = $command;
		$tmp .= "-" . $condition;
		$tmp .= ":" . $sub_uid;
		$PavgNew = $Pavg-$Pavg2;
		$RerrNew = calcrerr($tmp, $Pavg, $Rerr, $Pavg2, $Rerr2, $PavgNew);
		pline($itsy, $tmp, $PavgNew, $RerrNew);

		$tmp = $command;
		$tmp .= "-" . $condition;
		$tmp .= ":" . $rem_uid;
		pline($itsy, $tmp, $Pavg2, $Rerr2);
	    }
	}
    }
}

&arguments();

FILE1:

if ($#ARGV != 1) {
    &usage();
}

FILE2:

$infile1 = shift @ARGV;
$infile2 = shift @ARGV;
open(FILE1, $infile1) || (die "Can't open $infile1: $!\n");
open(FILE2, $infile2) || (die "Can't open $infile2: $!\n");
    
processit(FILE1, FILE2);
close(FILE1);
close(FILE2);

